# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._file_link import FileLink
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._file_link_create_params import FileLinkCreateParams
    from stripe.params._file_link_list_params import FileLinkListParams
    from stripe.params._file_link_retrieve_params import FileLinkRetrieveParams
    from stripe.params._file_link_update_params import FileLinkUpdateParams


class FileLinkService(StripeService):
    def list(
        self,
        params: Optional["FileLinkListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FileLink]":
        """
        Returns a list of file links.
        """
        return cast(
            "ListObject[FileLink]",
            self._request(
                "get",
                "/v1/file_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FileLinkListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FileLink]":
        """
        Returns a list of file links.
        """
        return cast(
            "ListObject[FileLink]",
            await self._request_async(
                "get",
                "/v1/file_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "FileLinkCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Creates a new file link object.
        """
        return cast(
            "FileLink",
            self._request(
                "post",
                "/v1/file_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FileLinkCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Creates a new file link object.
        """
        return cast(
            "FileLink",
            await self._request_async(
                "post",
                "/v1/file_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        link: str,
        params: Optional["FileLinkRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Retrieves the file link with the given ID.
        """
        return cast(
            "FileLink",
            self._request(
                "get",
                "/v1/file_links/{link}".format(link=sanitize_id(link)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        link: str,
        params: Optional["FileLinkRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Retrieves the file link with the given ID.
        """
        return cast(
            "FileLink",
            await self._request_async(
                "get",
                "/v1/file_links/{link}".format(link=sanitize_id(link)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        link: str,
        params: Optional["FileLinkUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Updates an existing file link object. Expired links can no longer be updated.
        """
        return cast(
            "FileLink",
            self._request(
                "post",
                "/v1/file_links/{link}".format(link=sanitize_id(link)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        link: str,
        params: Optional["FileLinkUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FileLink":
        """
        Updates an existing file link object. Expired links can no longer be updated.
        """
        return cast(
            "FileLink",
            await self._request_async(
                "post",
                "/v1/file_links/{link}".format(link=sanitize_id(link)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._dispute import Dispute
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._dispute_close_params import DisputeCloseParams
    from stripe.params._dispute_list_params import DisputeListParams
    from stripe.params._dispute_retrieve_params import DisputeRetrieveParams
    from stripe.params._dispute_update_params import DisputeUpdateParams


class DisputeService(StripeService):
    def list(
        self,
        params: Optional["DisputeListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Dispute]":
        """
        Returns a list of your disputes.
        """
        return cast(
            "ListObject[Dispute]",
            self._request(
                "get",
                "/v1/disputes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["DisputeListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Dispute]":
        """
        Returns a list of your disputes.
        """
        return cast(
            "ListObject[Dispute]",
            await self._request_async(
                "get",
                "/v1/disputes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        dispute: str,
        params: Optional["DisputeRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        Retrieves the dispute with the given ID.
        """
        return cast(
            "Dispute",
            self._request(
                "get",
                "/v1/disputes/{dispute}".format(dispute=sanitize_id(dispute)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        dispute: str,
        params: Optional["DisputeRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        Retrieves the dispute with the given ID.
        """
        return cast(
            "Dispute",
            await self._request_async(
                "get",
                "/v1/disputes/{dispute}".format(dispute=sanitize_id(dispute)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        dispute: str,
        params: Optional["DisputeUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        When you get a dispute, contacting your customer is always the best first step. If that doesn't work, you can submit evidence to help us resolve the dispute in your favor. You can do this in your [dashboard](https://dashboard.stripe.com/disputes), but if you prefer, you can use the API to submit evidence programmatically.

        Depending on your dispute type, different evidence fields will give you a better chance of winning your dispute. To figure out which evidence fields to provide, see our [guide to dispute types](https://docs.stripe.com/docs/disputes/categories).
        """
        return cast(
            "Dispute",
            self._request(
                "post",
                "/v1/disputes/{dispute}".format(dispute=sanitize_id(dispute)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        dispute: str,
        params: Optional["DisputeUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        When you get a dispute, contacting your customer is always the best first step. If that doesn't work, you can submit evidence to help us resolve the dispute in your favor. You can do this in your [dashboard](https://dashboard.stripe.com/disputes), but if you prefer, you can use the API to submit evidence programmatically.

        Depending on your dispute type, different evidence fields will give you a better chance of winning your dispute. To figure out which evidence fields to provide, see our [guide to dispute types](https://docs.stripe.com/docs/disputes/categories).
        """
        return cast(
            "Dispute",
            await self._request_async(
                "post",
                "/v1/disputes/{dispute}".format(dispute=sanitize_id(dispute)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def close(
        self,
        dispute: str,
        params: Optional["DisputeCloseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        Closing the dispute for a charge indicates that you do not have any evidence to submit and are essentially dismissing the dispute (accepting it), acknowledging it as lost.

        The status of the dispute will change from needs_response to lost. Closing a dispute is irreversible.
        """
        return cast(
            "Dispute",
            self._request(
                "post",
                "/v1/disputes/{dispute}/close".format(
                    dispute=sanitize_id(dispute),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def close_async(
        self,
        dispute: str,
        params: Optional["DisputeCloseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Dispute":
        """
        Closing the dispute for a charge indicates that you do not have any evidence to submit and are essentially dismissing the dispute (accepting it), acknowledging it as lost.

        The status of the dispute will change from needs_response to lost. Closing a dispute is irreversible.
        """
        return cast(
            "Dispute",
            await self._request_async(
                "post",
                "/v1/disputes/{dispute}/close".format(
                    dispute=sanitize_id(dispute),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional, Union
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.climate._supplier_list_params import SupplierListParams
    from stripe.params.climate._supplier_retrieve_params import (
        SupplierRetrieveParams,
    )


class Supplier(ListableAPIResource["Supplier"]):
    """
    A supplier of carbon removal.
    """

    OBJECT_NAME: ClassVar[Literal["climate.supplier"]] = "climate.supplier"

    class Location(StripeObject):
        city: Optional[str]
        """
        The city where the supplier is located.
        """
        country: str
        """
        Two-letter ISO code representing the country where the supplier is located.
        """
        latitude: Optional[float]
        """
        The geographic latitude where the supplier is located.
        """
        longitude: Optional[float]
        """
        The geographic longitude where the supplier is located.
        """
        region: Optional[str]
        """
        The state/county/province/region where the supplier is located.
        """

    id: str
    """
    Unique identifier for the object.
    """
    info_url: str
    """
    Link to a webpage to learn more about the supplier.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    locations: List[Location]
    """
    The locations in which this supplier operates.
    """
    name: str
    """
    Name of this carbon removal supplier.
    """
    object: Literal["climate.supplier"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    removal_pathway: Union[
        Literal[
            "biomass_carbon_removal_and_storage",
            "direct_air_capture",
            "enhanced_weathering",
            "marine_carbon_removal",
        ],
        str,
    ]
    """
    The scientific pathway used for carbon removal.
    """

    @classmethod
    def list(
        cls, **params: Unpack["SupplierListParams"]
    ) -> ListObject["Supplier"]:
        """
        Lists all available Climate supplier objects.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["SupplierListParams"]
    ) -> ListObject["Supplier"]:
        """
        Lists all available Climate supplier objects.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["SupplierRetrieveParams"]
    ) -> "Supplier":
        """
        Retrieves a Climate supplier object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["SupplierRetrieveParams"]
    ) -> "Supplier":
        """
        Retrieves a Climate supplier object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"locations": Location}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class FileListParams(RequestOptions):
    created: NotRequired["FileListParamsCreated|int"]
    """
    Only return files that were created during the given date interval.
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    purpose: NotRequired[
        "Literal['account_requirement', 'additional_verification', 'business_icon', 'business_logo', 'customer_signature', 'dispute_evidence', 'document_provider_identity_document', 'finance_report_run', 'financial_account_statement', 'identity_document', 'identity_document_downloadable', 'issuing_regulatory_reporting', 'pci_document', 'platform_terms_of_service', 'selfie', 'sigma_scheduled_query', 'tax_document_user_upload', 'terminal_android_apk', 'terminal_reader_splashscreen', 'terminal_wifi_certificate', 'terminal_wifi_private_key']|str"
    ]
    """
    Filter queries by the file purpose. If you don't provide a purpose, the queries return unfiltered files.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """


class FileListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Minimum value to filter by (exclusive)
    """
    gte: NotRequired[int]
    """
    Minimum value to filter by (inclusive)
    """
    lt: NotRequired[int]
    """
    Maximum value to filter by (exclusive)
    """
    lte: NotRequired[int]
    """
    Maximum value to filter by (inclusive)
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._encode import _coerce_v2_params
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core._account_token_create_params import (
        AccountTokenCreateParams,
    )
    from stripe.params.v2.core._account_token_retrieve_params import (
        AccountTokenRetrieveParams,
    )
    from stripe.v2.core._account_token import AccountToken


class AccountTokenService(StripeService):
    def create(
        self,
        params: Optional["AccountTokenCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountToken":
        """
        Create an account token with a publishable key and pass it to the Accounts v2 API to
        create or update an account without its data touching your server.
        Learn more about [account tokens](https://docs.stripe.com/connect/account-tokens).
        In live mode, you can only create account tokens with your application's publishable key.
        In test mode, you can create account tokens with your secret key or publishable key.
        """
        return cast(
            "AccountToken",
            self._request(
                "post",
                "/v2/core/account_tokens",
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["AccountTokenCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountToken":
        """
        Create an account token with a publishable key and pass it to the Accounts v2 API to
        create or update an account without its data touching your server.
        Learn more about [account tokens](https://docs.stripe.com/connect/account-tokens).
        In live mode, you can only create account tokens with your application's publishable key.
        In test mode, you can create account tokens with your secret key or publishable key.
        """
        return cast(
            "AccountToken",
            await self._request_async(
                "post",
                "/v2/core/account_tokens",
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["AccountTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountToken":
        """
        Retrieves an Account Token.
        """
        return cast(
            "AccountToken",
            self._request(
                "get",
                "/v2/core/account_tokens/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["AccountTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountToken":
        """
        Retrieves an Account Token.
        """
        return cast(
            "AccountToken",
            await self._request_async(
                "get",
                "/v2/core/account_tokens/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._encode import _coerce_v2_params
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core._account_close_params import AccountCloseParams
    from stripe.params.v2.core._account_create_params import (
        AccountCreateParams,
    )
    from stripe.params.v2.core._account_list_params import AccountListParams
    from stripe.params.v2.core._account_retrieve_params import (
        AccountRetrieveParams,
    )
    from stripe.params.v2.core._account_update_params import (
        AccountUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.core._account import Account
    from stripe.v2.core.accounts._person_service import PersonService
    from stripe.v2.core.accounts._person_token_service import (
        PersonTokenService,
    )

_subservices = {
    "persons": ["stripe.v2.core.accounts._person_service", "PersonService"],
    "person_tokens": [
        "stripe.v2.core.accounts._person_token_service",
        "PersonTokenService",
    ],
}


class AccountService(StripeService):
    persons: "PersonService"
    person_tokens: "PersonTokenService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["AccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Account]":
        """
        Returns a list of Accounts.
        """
        return cast(
            "ListObject[Account]",
            self._request(
                "get",
                "/v2/core/accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["AccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Account]":
        """
        Returns a list of Accounts.
        """
        return cast(
            "ListObject[Account]",
            await self._request_async(
                "get",
                "/v2/core/accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["AccountCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Create an Account that represents a company, individual, or other entity that your business interacts with. Accounts contain identifying information about the entity, and configurations that store the features an account has access to. An account can be configured as any or all of the following configurations: Customer, Merchant and/or Recipient.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v2/core/accounts",
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["AccountCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Create an Account that represents a company, individual, or other entity that your business interacts with. Accounts contain identifying information about the entity, and configurations that store the features an account has access to. An account can be configured as any or all of the following configurations: Customer, Merchant and/or Recipient.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v2/core/accounts",
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["AccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Retrieves the details of an Account.
        """
        return cast(
            "Account",
            self._request(
                "get",
                "/v2/core/accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["AccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Retrieves the details of an Account.
        """
        return cast(
            "Account",
            await self._request_async(
                "get",
                "/v2/core/accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["AccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Updates the details of an Account.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v2/core/accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["AccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Updates the details of an Account.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v2/core/accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=_coerce_v2_params(
                    params,
                    {
                        "identity": {
                            "individual": {
                                "relationship": {
                                    "percent_ownership": "decimal_string",
                                },
                            },
                        },
                    },
                ),
                options=options,
            ),
        )

    def close(
        self,
        id: str,
        params: Optional["AccountCloseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Removes access to the Account and its associated resources. Closed Accounts can no longer be operated on, but limited information can still be retrieved through the API in order to be able to track their history.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v2/core/accounts/{id}/close".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def close_async(
        self,
        id: str,
        params: Optional["AccountCloseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Removes access to the Account and its associated resources. Closed Accounts can no longer be operated on, but limited information can still be retrieved through the API in order to be able to track their history.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v2/core/accounts/{id}/close".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional, Union
from typing_extensions import Literal


class Plan(StripeObject):
    """
    ReservePlans are used to automatically place holds on a merchant's funds until the plan expires. It takes a portion of each incoming Charge (including those resulting from a Transfer from a platform account).
    """

    OBJECT_NAME: ClassVar[Literal["reserve.plan"]] = "reserve.plan"

    class FixedRelease(StripeObject):
        release_after: int
        """
        The time after which all reserved funds are requested for release.
        """
        scheduled_release: int
        """
        The time at which reserved funds are scheduled for release, automatically set to midnight UTC of the day after `release_after`.
        """

    class RollingRelease(StripeObject):
        days_after_charge: int
        """
        The number of days to reserve funds before releasing.
        """
        expires_on: Optional[int]
        """
        The time at which the ReservePlan expires.
        """

    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    created_by: Union[Literal["application", "stripe"], str]
    """
    Indicates which party created this ReservePlan.
    """
    currency: Optional[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies). An unset currency indicates that the plan applies to all currencies.
    """
    disabled_at: Optional[int]
    """
    Time at which the ReservePlan was disabled.
    """
    fixed_release: Optional[FixedRelease]
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["reserve.plan"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    percent: int
    """
    The percent of each Charge to reserve.
    """
    rolling_release: Optional[RollingRelease]
    status: Union[Literal["active", "disabled", "expired"], str]
    """
    The current status of the ReservePlan. The ReservePlan only affects charges if it is `active`.
    """
    type: Union[Literal["fixed_release", "rolling_release"], str]
    """
    The type of the ReservePlan.
    """
    _inner_class_types = {
        "fixed_release": FixedRelease,
        "rolling_release": RollingRelease,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class QuoteFinalizeQuoteParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    expires_at: NotRequired[int]
    """
    A future timestamp on which the quote will be canceled if in `open` or `draft` status. Measured in seconds since the Unix epoch.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class CardListParams(RequestOptions):
    cardholder: NotRequired[str]
    """
    Only return cards belonging to the Cardholder with the provided ID.
    """
    created: NotRequired["CardListParamsCreated|int"]
    """
    Only return cards that were issued during the given date interval.
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    exp_month: NotRequired[int]
    """
    Only return cards that have the given expiration month.
    """
    exp_year: NotRequired[int]
    """
    Only return cards that have the given expiration year.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    last4: NotRequired[str]
    """
    Only return cards that have the given last four digits.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    personalization_design: NotRequired[str]
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """
    status: NotRequired["Literal['active', 'canceled', 'inactive']|str"]
    """
    Only return cards that have the given status. One of `active`, `inactive`, or `canceled`.
    """
    type: NotRequired["Literal['physical', 'virtual']|str"]
    """
    Only return cards that have the given type. One of `virtual` or `physical`.
    """


class CardListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Minimum value to filter by (exclusive)
    """
    gte: NotRequired[int]
    """
    Minimum value to filter by (inclusive)
    """
    lt: NotRequired[int]
    """
    Maximum value to filter by (exclusive)
    """
    lte: NotRequired[int]
    """
    Maximum value to filter by (inclusive)
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional, Union
from typing_extensions import Literal


class Authorization(StripeObject):
    """
    An Authorization represents the set of credentials used to connect a group of Financial Connections Accounts.
    """

    OBJECT_NAME: ClassVar[Literal["financial_connections.authorization"]] = (
        "financial_connections.authorization"
    )

    class StatusDetails(StripeObject):
        class Active(StripeObject):
            action: Union[Literal["none", "relink_required"], str]
            """
            The action (if any) to proactively relink the Authorization.
            """
            expected_deactivation_date: int
            """
            When the Authorization is expected to become inactive, if applicable.
            """

        class Inactive(StripeObject):
            action: Union[Literal["none", "relink_required"], str]
            """
            The action (if any) to relink the inactive Authorization.
            """

        active: Optional[Active]
        inactive: Optional[Inactive]
        _inner_class_types = {"active": Active, "inactive": Inactive}

    id: str
    """
    Unique identifier for the object.
    """
    institution_name: str
    """
    The name of the institution that this authorization belongs to.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["financial_connections.authorization"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Union[Literal["active", "inactive"], str]
    """
    The status of the connection to the Authorization.
    """
    status_details: StatusDetails
    _inner_class_types = {"status_details": StatusDetails}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class ValueListCreateParams(RequestOptions):
    alias: str
    """
    The name of the value list for use in rules.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    item_type: NotRequired[
        "Literal['account', 'card_bin', 'card_fingerprint', 'case_sensitive_string', 'country', 'crypto_fingerprint', 'customer_id', 'email', 'ip_address', 'sepa_debit_fingerprint', 'string', 'us_bank_account_fingerprint']|str"
    ]
    """
    Type of the items in the value list. One of `card_fingerprint`, `card_bin`, `crypto_fingerprint`, `email`, `ip_address`, `country`, `string`, `case_sensitive_string`, `customer_id`, `account`, `sepa_debit_fingerprint`, or `us_bank_account_fingerprint`. Use `string` if the item type is unknown or mixed.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: str
    """
    The human-readable name of the value list.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.radar._payment_evaluation_create_params import (
        PaymentEvaluationCreateParams,
    )
    from stripe.radar._payment_evaluation import PaymentEvaluation


class PaymentEvaluationService(StripeService):
    def create(
        self,
        params: "PaymentEvaluationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentEvaluation":
        """
        Request a Radar API fraud risk score from Stripe for a payment before sending it for external processor authorization.
        """
        return cast(
            "PaymentEvaluation",
            self._request(
                "post",
                "/v1/radar/payment_evaluations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PaymentEvaluationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentEvaluation":
        """
        Request a Radar API fraud risk score from Stripe for a payment before sending it for external processor authorization.
        """
        return cast(
            "PaymentEvaluation",
            await self._request_async(
                "post",
                "/v1/radar/payment_evaluations",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class TransactionListParams(RequestOptions):
    card: NotRequired[str]
    """
    Only return transactions that belong to the given card.
    """
    cardholder: NotRequired[str]
    """
    Only return transactions that belong to the given cardholder.
    """
    created: NotRequired["TransactionListParamsCreated|int"]
    """
    Only return transactions that were created during the given date interval.
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """
    type: NotRequired["Literal['capture', 'refund']|str"]
    """
    Only return transactions that have the given type. One of `capture` or `refund`.
    """


class TransactionListParamsCreated(TypedDict):
    gt: NotRequired[int]
    """
    Minimum value to filter by (exclusive)
    """
    gte: NotRequired[int]
    """
    Minimum value to filter by (inclusive)
    """
    lt: NotRequired[int]
    """
    Maximum value to filter by (exclusive)
    """
    lte: NotRequired[int]
    """
    Maximum value to filter by (inclusive)
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.treasury._outbound_transfer_cancel_params import (
        OutboundTransferCancelParams,
    )
    from stripe.params.treasury._outbound_transfer_create_params import (
        OutboundTransferCreateParams,
    )
    from stripe.params.treasury._outbound_transfer_list_params import (
        OutboundTransferListParams,
    )
    from stripe.params.treasury._outbound_transfer_retrieve_params import (
        OutboundTransferRetrieveParams,
    )
    from stripe.treasury._outbound_transfer import OutboundTransfer


class OutboundTransferService(StripeService):
    def list(
        self,
        params: "OutboundTransferListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundTransfer]":
        """
        Returns a list of OutboundTransfers sent from the specified FinancialAccount.
        """
        return cast(
            "ListObject[OutboundTransfer]",
            self._request(
                "get",
                "/v1/treasury/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "OutboundTransferListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundTransfer]":
        """
        Returns a list of OutboundTransfers sent from the specified FinancialAccount.
        """
        return cast(
            "ListObject[OutboundTransfer]",
            await self._request_async(
                "get",
                "/v1/treasury/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "OutboundTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Creates an OutboundTransfer.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/treasury/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "OutboundTransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Creates an OutboundTransfer.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/treasury/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Retrieves the details of an existing OutboundTransfer by passing the unique OutboundTransfer ID from either the OutboundTransfer creation request or OutboundTransfer list.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "get",
                "/v1/treasury/outbound_transfers/{outbound_transfer}".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Retrieves the details of an existing OutboundTransfer by passing the unique OutboundTransfer ID from either the OutboundTransfer creation request or OutboundTransfer list.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "get",
                "/v1/treasury/outbound_transfers/{outbound_transfer}".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        An OutboundTransfer can be canceled if the funds have not yet been paid out.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/treasury/outbound_transfers/{outbound_transfer}/cancel".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        An OutboundTransfer can be canceled if the funds have not yet been paid out.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/treasury/outbound_transfers/{outbound_transfer}/cancel".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.test_helpers.issuing._authorization_capture_params import (
        AuthorizationCaptureParams as AuthorizationCaptureParams,
        AuthorizationCaptureParamsPurchaseDetails as AuthorizationCaptureParamsPurchaseDetails,
        AuthorizationCaptureParamsPurchaseDetailsFleet as AuthorizationCaptureParamsPurchaseDetailsFleet,
        AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData as AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax as AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax,
        AuthorizationCaptureParamsPurchaseDetailsFlight as AuthorizationCaptureParamsPurchaseDetailsFlight,
        AuthorizationCaptureParamsPurchaseDetailsFlightSegment as AuthorizationCaptureParamsPurchaseDetailsFlightSegment,
        AuthorizationCaptureParamsPurchaseDetailsFuel as AuthorizationCaptureParamsPurchaseDetailsFuel,
        AuthorizationCaptureParamsPurchaseDetailsLodging as AuthorizationCaptureParamsPurchaseDetailsLodging,
        AuthorizationCaptureParamsPurchaseDetailsReceipt as AuthorizationCaptureParamsPurchaseDetailsReceipt,
    )
    from stripe.params.test_helpers.issuing._authorization_create_params import (
        AuthorizationCreateParams as AuthorizationCreateParams,
        AuthorizationCreateParamsAmountDetails as AuthorizationCreateParamsAmountDetails,
        AuthorizationCreateParamsFleet as AuthorizationCreateParamsFleet,
        AuthorizationCreateParamsFleetCardholderPromptData as AuthorizationCreateParamsFleetCardholderPromptData,
        AuthorizationCreateParamsFleetReportedBreakdown as AuthorizationCreateParamsFleetReportedBreakdown,
        AuthorizationCreateParamsFleetReportedBreakdownFuel as AuthorizationCreateParamsFleetReportedBreakdownFuel,
        AuthorizationCreateParamsFleetReportedBreakdownNonFuel as AuthorizationCreateParamsFleetReportedBreakdownNonFuel,
        AuthorizationCreateParamsFleetReportedBreakdownTax as AuthorizationCreateParamsFleetReportedBreakdownTax,
        AuthorizationCreateParamsFuel as AuthorizationCreateParamsFuel,
        AuthorizationCreateParamsMerchantData as AuthorizationCreateParamsMerchantData,
        AuthorizationCreateParamsNetworkData as AuthorizationCreateParamsNetworkData,
        AuthorizationCreateParamsRiskAssessment as AuthorizationCreateParamsRiskAssessment,
        AuthorizationCreateParamsRiskAssessmentCardTestingRisk as AuthorizationCreateParamsRiskAssessmentCardTestingRisk,
        AuthorizationCreateParamsRiskAssessmentFraudRisk as AuthorizationCreateParamsRiskAssessmentFraudRisk,
        AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk as AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk,
        AuthorizationCreateParamsVerificationData as AuthorizationCreateParamsVerificationData,
        AuthorizationCreateParamsVerificationDataAuthenticationExemption as AuthorizationCreateParamsVerificationDataAuthenticationExemption,
        AuthorizationCreateParamsVerificationDataThreeDSecure as AuthorizationCreateParamsVerificationDataThreeDSecure,
    )
    from stripe.params.test_helpers.issuing._authorization_expire_params import (
        AuthorizationExpireParams as AuthorizationExpireParams,
    )
    from stripe.params.test_helpers.issuing._authorization_finalize_amount_params import (
        AuthorizationFinalizeAmountParams as AuthorizationFinalizeAmountParams,
        AuthorizationFinalizeAmountParamsFleet as AuthorizationFinalizeAmountParamsFleet,
        AuthorizationFinalizeAmountParamsFleetCardholderPromptData as AuthorizationFinalizeAmountParamsFleetCardholderPromptData,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdown as AuthorizationFinalizeAmountParamsFleetReportedBreakdown,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel as AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel as AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel,
        AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax as AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax,
        AuthorizationFinalizeAmountParamsFuel as AuthorizationFinalizeAmountParamsFuel,
    )
    from stripe.params.test_helpers.issuing._authorization_increment_params import (
        AuthorizationIncrementParams as AuthorizationIncrementParams,
    )
    from stripe.params.test_helpers.issuing._authorization_respond_params import (
        AuthorizationRespondParams as AuthorizationRespondParams,
    )
    from stripe.params.test_helpers.issuing._authorization_reverse_params import (
        AuthorizationReverseParams as AuthorizationReverseParams,
    )
    from stripe.params.test_helpers.issuing._card_deliver_card_params import (
        CardDeliverCardParams as CardDeliverCardParams,
    )
    from stripe.params.test_helpers.issuing._card_fail_card_params import (
        CardFailCardParams as CardFailCardParams,
    )
    from stripe.params.test_helpers.issuing._card_return_card_params import (
        CardReturnCardParams as CardReturnCardParams,
    )
    from stripe.params.test_helpers.issuing._card_ship_card_params import (
        CardShipCardParams as CardShipCardParams,
    )
    from stripe.params.test_helpers.issuing._card_submit_card_params import (
        CardSubmitCardParams as CardSubmitCardParams,
    )
    from stripe.params.test_helpers.issuing._personalization_design_activate_params import (
        PersonalizationDesignActivateParams as PersonalizationDesignActivateParams,
    )
    from stripe.params.test_helpers.issuing._personalization_design_deactivate_params import (
        PersonalizationDesignDeactivateParams as PersonalizationDesignDeactivateParams,
    )
    from stripe.params.test_helpers.issuing._personalization_design_reject_params import (
        PersonalizationDesignRejectParams as PersonalizationDesignRejectParams,
        PersonalizationDesignRejectParamsRejectionReasons as PersonalizationDesignRejectParamsRejectionReasons,
    )
    from stripe.params.test_helpers.issuing._transaction_create_force_capture_params import (
        TransactionCreateForceCaptureParams as TransactionCreateForceCaptureParams,
        TransactionCreateForceCaptureParamsMerchantData as TransactionCreateForceCaptureParamsMerchantData,
        TransactionCreateForceCaptureParamsPurchaseDetails as TransactionCreateForceCaptureParamsPurchaseDetails,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleet as TransactionCreateForceCaptureParamsPurchaseDetailsFleet,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData as TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax as TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax,
        TransactionCreateForceCaptureParamsPurchaseDetailsFlight as TransactionCreateForceCaptureParamsPurchaseDetailsFlight,
        TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment as TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment,
        TransactionCreateForceCaptureParamsPurchaseDetailsFuel as TransactionCreateForceCaptureParamsPurchaseDetailsFuel,
        TransactionCreateForceCaptureParamsPurchaseDetailsLodging as TransactionCreateForceCaptureParamsPurchaseDetailsLodging,
        TransactionCreateForceCaptureParamsPurchaseDetailsReceipt as TransactionCreateForceCaptureParamsPurchaseDetailsReceipt,
    )
    from stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params import (
        TransactionCreateUnlinkedRefundParams as TransactionCreateUnlinkedRefundParams,
        TransactionCreateUnlinkedRefundParamsMerchantData as TransactionCreateUnlinkedRefundParamsMerchantData,
        TransactionCreateUnlinkedRefundParamsPurchaseDetails as TransactionCreateUnlinkedRefundParamsPurchaseDetails,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel as TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging as TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging,
        TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt as TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt,
    )
    from stripe.params.test_helpers.issuing._transaction_refund_params import (
        TransactionRefundParams as TransactionRefundParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "AuthorizationCaptureParams": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetails": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleet": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFlight": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFlightSegment": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsFuel": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsLodging": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCaptureParamsPurchaseDetailsReceipt": (
        "stripe.params.test_helpers.issuing._authorization_capture_params",
        False,
    ),
    "AuthorizationCreateParams": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsAmountDetails": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleet": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetCardholderPromptData": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdown": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownFuel": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownNonFuel": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFleetReportedBreakdownTax": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsFuel": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsMerchantData": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsNetworkData": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessment": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentCardTestingRisk": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentFraudRisk": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsRiskAssessmentMerchantDisputeRisk": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationData": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationDataAuthenticationExemption": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationCreateParamsVerificationDataThreeDSecure": (
        "stripe.params.test_helpers.issuing._authorization_create_params",
        False,
    ),
    "AuthorizationExpireParams": (
        "stripe.params.test_helpers.issuing._authorization_expire_params",
        False,
    ),
    "AuthorizationFinalizeAmountParams": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleet": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetCardholderPromptData": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdown": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownFuel": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownNonFuel": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFleetReportedBreakdownTax": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationFinalizeAmountParamsFuel": (
        "stripe.params.test_helpers.issuing._authorization_finalize_amount_params",
        False,
    ),
    "AuthorizationIncrementParams": (
        "stripe.params.test_helpers.issuing._authorization_increment_params",
        False,
    ),
    "AuthorizationRespondParams": (
        "stripe.params.test_helpers.issuing._authorization_respond_params",
        False,
    ),
    "AuthorizationReverseParams": (
        "stripe.params.test_helpers.issuing._authorization_reverse_params",
        False,
    ),
    "CardDeliverCardParams": (
        "stripe.params.test_helpers.issuing._card_deliver_card_params",
        False,
    ),
    "CardFailCardParams": (
        "stripe.params.test_helpers.issuing._card_fail_card_params",
        False,
    ),
    "CardReturnCardParams": (
        "stripe.params.test_helpers.issuing._card_return_card_params",
        False,
    ),
    "CardShipCardParams": (
        "stripe.params.test_helpers.issuing._card_ship_card_params",
        False,
    ),
    "CardSubmitCardParams": (
        "stripe.params.test_helpers.issuing._card_submit_card_params",
        False,
    ),
    "PersonalizationDesignActivateParams": (
        "stripe.params.test_helpers.issuing._personalization_design_activate_params",
        False,
    ),
    "PersonalizationDesignDeactivateParams": (
        "stripe.params.test_helpers.issuing._personalization_design_deactivate_params",
        False,
    ),
    "PersonalizationDesignRejectParams": (
        "stripe.params.test_helpers.issuing._personalization_design_reject_params",
        False,
    ),
    "PersonalizationDesignRejectParamsRejectionReasons": (
        "stripe.params.test_helpers.issuing._personalization_design_reject_params",
        False,
    ),
    "TransactionCreateForceCaptureParams": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsMerchantData": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetails": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleet": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFlight": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFlightSegment": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsLodging": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateForceCaptureParamsPurchaseDetailsReceipt": (
        "stripe.params.test_helpers.issuing._transaction_create_force_capture_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParams": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsMerchantData": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetails": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleet": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetCardholderPromptData": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdown": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownNonFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFleetReportedBreakdownTax": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlight": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFlightSegment": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsFuel": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsLodging": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionCreateUnlinkedRefundParamsPurchaseDetailsReceipt": (
        "stripe.params.test_helpers.issuing._transaction_create_unlinked_refund_params",
        False,
    ),
    "TransactionRefundParams": (
        "stripe.params.test_helpers.issuing._transaction_refund_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, TypedDict


class MeterEventAdjustmentCreateParams(TypedDict):
    cancel: "MeterEventAdjustmentCreateParamsCancel"
    """
    Specifies which event to cancel.
    """
    event_name: str
    """
    The name of the meter event. Corresponds with the `event_name` field on a meter.
    """
    type: Literal["cancel"]
    """
    Specifies the type of cancellation. Currently supports canceling a single event.
    """


class MeterEventAdjustmentCreateParamsCancel(TypedDict):
    identifier: str
    """
    The identifier that was originally assigned to the meter event. You can only cancel events within 24 hours of Stripe receiving them.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class CalculationCreateParams(RequestOptions):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: NotRequired[str]
    """
    The ID of an existing customer to use for this calculation. If provided, the customer's address and tax IDs are copied to `customer_details`.
    """
    customer_details: NotRequired["CalculationCreateParamsCustomerDetails"]
    """
    Details about the customer, including address and tax IDs.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    line_items: List["CalculationCreateParamsLineItem"]
    """
    A list of items the customer is purchasing. You can pass up to 100 line items, or 1,000 if your account has an increased limit.
    """
    ship_from_details: NotRequired["CalculationCreateParamsShipFromDetails"]
    """
    Details about the address from which the goods are being shipped.
    """
    shipping_cost: NotRequired["CalculationCreateParamsShippingCost"]
    """
    Shipping cost details to be used for the calculation.
    """
    tax_date: NotRequired[int]
    """
    The calculation uses the tax rules and rates that are in effect at this timestamp. You can use a date up to 31 days in the past or up to 31 days in the future. If you use a future date, Stripe doesn't guarantee that the expected tax rules and rate being used match the actual rules and rate that will be in effect on that date. We deploy tax changes before their effective date, but not within a fixed window.
    """


class CalculationCreateParamsCustomerDetails(TypedDict):
    address: NotRequired["CalculationCreateParamsCustomerDetailsAddress"]
    """
    The customer's postal address (for example, home or business location).
    """
    address_source: NotRequired["Literal['billing', 'shipping']|str"]
    """
    The type of customer address provided.
    """
    ip_address: NotRequired[str]
    """
    The customer's IP address (IPv4 or IPv6).
    """
    tax_ids: NotRequired[List["CalculationCreateParamsCustomerDetailsTaxId"]]
    """
    The customer's tax IDs. Stripe Tax might consider a transaction with applicable tax IDs to be B2B, which might affect the tax calculation result. Stripe Tax doesn't validate tax IDs for correctness.
    """
    taxability_override: NotRequired[
        "Literal['customer_exempt', 'none', 'reverse_charge']|str"
    ]
    """
    Overrides the tax calculation result to allow you to not collect tax from your customer. Use this if you've manually checked your customer's tax exemptions. Prefer providing the customer's `tax_ids` where possible, which automatically determines whether `reverse_charge` applies.
    """


class CalculationCreateParamsCustomerDetailsAddress(TypedDict):
    city: NotRequired["Literal['']|str"]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired["Literal['']|str"]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired["Literal['']|str"]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired["Literal['']|str"]
    """
    ZIP or postal code.
    """
    state: NotRequired["Literal['']|str"]
    """
    State, county, province, or region. We recommend sending [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code value when possible.
    """


class CalculationCreateParamsCustomerDetailsTaxId(TypedDict):
    type: Union[
        Literal[
            "ad_nrt",
            "ae_trn",
            "al_tin",
            "am_tin",
            "ao_tin",
            "ar_cuit",
            "au_abn",
            "au_arn",
            "aw_tin",
            "az_tin",
            "ba_tin",
            "bb_tin",
            "bd_bin",
            "bf_ifu",
            "bg_uic",
            "bh_vat",
            "bj_ifu",
            "bo_tin",
            "br_cnpj",
            "br_cpf",
            "bs_tin",
            "by_tin",
            "ca_bn",
            "ca_gst_hst",
            "ca_pst_bc",
            "ca_pst_mb",
            "ca_pst_sk",
            "ca_qst",
            "cd_nif",
            "ch_uid",
            "ch_vat",
            "cl_tin",
            "cm_niu",
            "cn_tin",
            "co_nit",
            "cr_tin",
            "cv_nif",
            "de_stn",
            "do_rcn",
            "ec_ruc",
            "eg_tin",
            "es_cif",
            "et_tin",
            "eu_oss_vat",
            "eu_vat",
            "fo_vat",
            "gb_vat",
            "ge_vat",
            "gi_tin",
            "gn_nif",
            "hk_br",
            "hr_oib",
            "hu_tin",
            "ic_nif",
            "id_npwp",
            "il_vat",
            "in_gst",
            "is_vat",
            "it_cf",
            "jp_cn",
            "jp_rn",
            "jp_trn",
            "ke_pin",
            "kg_tin",
            "kh_tin",
            "kr_brn",
            "kz_bin",
            "la_tin",
            "li_uid",
            "li_vat",
            "lk_vat",
            "ma_vat",
            "md_vat",
            "me_pib",
            "mk_vat",
            "mr_nif",
            "mx_rfc",
            "my_frp",
            "my_itn",
            "my_sst",
            "ng_tin",
            "no_vat",
            "no_voec",
            "np_pan",
            "nz_gst",
            "om_vat",
            "pe_ruc",
            "ph_tin",
            "pl_nip",
            "py_ruc",
            "ro_tin",
            "rs_pib",
            "ru_inn",
            "ru_kpp",
            "sa_vat",
            "sg_gst",
            "sg_uen",
            "si_tin",
            "sn_ninea",
            "sr_fin",
            "sv_nit",
            "th_vat",
            "tj_tin",
            "tr_tin",
            "tw_vat",
            "tz_vat",
            "ua_vat",
            "ug_tin",
            "us_ein",
            "uy_ruc",
            "uz_tin",
            "uz_vat",
            "ve_rif",
            "vn_tin",
            "za_vat",
            "zm_tin",
            "zw_tin",
        ],
        str,
    ]
    """
    Type of the tax ID, one of `ad_nrt`, `ae_trn`, `al_tin`, `am_tin`, `ao_tin`, `ar_cuit`, `au_abn`, `au_arn`, `aw_tin`, `az_tin`, `ba_tin`, `bb_tin`, `bd_bin`, `bf_ifu`, `bg_uic`, `bh_vat`, `bj_ifu`, `bo_tin`, `br_cnpj`, `br_cpf`, `bs_tin`, `by_tin`, `ca_bn`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `ca_qst`, `cd_nif`, `ch_uid`, `ch_vat`, `cl_tin`, `cm_niu`, `cn_tin`, `co_nit`, `cr_tin`, `cv_nif`, `de_stn`, `do_rcn`, `ec_ruc`, `eg_tin`, `es_cif`, `et_tin`, `eu_oss_vat`, `eu_vat`, `fo_vat`, `gb_vat`, `ge_vat`, `gi_tin`, `gn_nif`, `hk_br`, `hr_oib`, `hu_tin`, `ic_nif`, `id_npwp`, `il_vat`, `in_gst`, `is_vat`, `it_cf`, `jp_cn`, `jp_rn`, `jp_trn`, `ke_pin`, `kg_tin`, `kh_tin`, `kr_brn`, `kz_bin`, `la_tin`, `li_uid`, `li_vat`, `lk_vat`, `ma_vat`, `md_vat`, `me_pib`, `mk_vat`, `mr_nif`, `mx_rfc`, `my_frp`, `my_itn`, `my_sst`, `ng_tin`, `no_vat`, `no_voec`, `np_pan`, `nz_gst`, `om_vat`, `pe_ruc`, `ph_tin`, `pl_nip`, `py_ruc`, `ro_tin`, `rs_pib`, `ru_inn`, `ru_kpp`, `sa_vat`, `sg_gst`, `sg_uen`, `si_tin`, `sn_ninea`, `sr_fin`, `sv_nit`, `th_vat`, `tj_tin`, `tr_tin`, `tw_vat`, `tz_vat`, `ua_vat`, `ug_tin`, `us_ein`, `uy_ruc`, `uz_tin`, `uz_vat`, `ve_rif`, `vn_tin`, `za_vat`, `zm_tin`, or `zw_tin`
    """
    value: str
    """
    Value of the tax ID.
    """


class CalculationCreateParamsLineItem(TypedDict):
    amount: int
    """
    A positive integer representing the line item's total price in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes are calculated on top of this amount.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    product: NotRequired[str]
    """
    If provided, the product's `tax_code` will be used as the line item's `tax_code`.
    """
    quantity: NotRequired[int]
    """
    The number of units of the item being purchased. Used to calculate the per-unit price from the total `amount` for the line. For example, if `amount=100` and `quantity=4`, the calculated unit price is 25.
    """
    reference: NotRequired[str]
    """
    A custom identifier for this line item, which must be unique across the line items in the calculation. The reference helps identify each line item in exported [tax reports](https://docs.stripe.com/tax/reports).
    """
    tax_behavior: NotRequired["Literal['exclusive', 'inclusive']|str"]
    """
    Specifies whether the `amount` includes taxes. Defaults to `exclusive`.
    """
    tax_code: NotRequired[str]
    """
    A [tax code](https://docs.stripe.com/tax/tax-categories) ID to use for this line item. If not provided, we will use the tax code from the provided `product` param. If neither `tax_code` nor `product` is provided, we will use the default tax code from your Tax Settings.
    """


class CalculationCreateParamsShipFromDetails(TypedDict):
    address: "CalculationCreateParamsShipFromDetailsAddress"
    """
    The address from which the goods are being shipped from.
    """


class CalculationCreateParamsShipFromDetailsAddress(TypedDict):
    city: NotRequired["Literal['']|str"]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired["Literal['']|str"]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired["Literal['']|str"]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired["Literal['']|str"]
    """
    ZIP or postal code.
    """
    state: NotRequired["Literal['']|str"]
    """
    State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
    """


class CalculationCreateParamsShippingCost(TypedDict):
    amount: NotRequired[int]
    """
    A positive integer in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units) representing the shipping charge. If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes are calculated on top of this amount.
    """
    shipping_rate: NotRequired[str]
    """
    If provided, the [shipping rate](https://docs.stripe.com/api/shipping_rates/object)'s `amount`, `tax_code` and `tax_behavior` are used. If you provide a shipping rate, then you cannot pass the `amount`, `tax_code`, or `tax_behavior` parameters.
    """
    tax_behavior: NotRequired["Literal['exclusive', 'inclusive']|str"]
    """
    Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes. Defaults to `exclusive`.
    """
    tax_code: NotRequired[str]
    """
    The [tax code](https://docs.stripe.com/tax/tax-categories) used to calculate tax on shipping. If not provided, the default shipping tax code from your [Tax Settings](https://dashboard.stripe.com/settings/tax) is used.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class TransferReversalCreateParams(TypedDict):
    amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) representing how much of this transfer to reverse. Can only reverse up to the unreversed amount remaining of the transfer. Partial transfer reversals are only allowed for transfers to Stripe Accounts. Defaults to the entire transfer amount.
    """
    description: NotRequired[str]
    """
    An arbitrary string which you can attach to a reversal object. This will be unset if you POST an empty value.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    refund_application_fee: NotRequired[bool]
    """
    Boolean indicating whether the application fee should be refunded when reversing this transfer. If a full transfer reversal is given, the full application fee will be refunded. Otherwise, the application fee will be refunded with an amount proportional to the amount of the transfer reversed.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class ReceivedCreditCreateParams(RequestOptions):
    amount: int
    """
    Amount (in cents) to be transferred.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    financial_account: str
    """
    The FinancialAccount to send funds to.
    """
    initiating_payment_method_details: NotRequired[
        "ReceivedCreditCreateParamsInitiatingPaymentMethodDetails"
    ]
    """
    Initiating payment method details for the object.
    """
    network: Union[Literal["ach", "us_domestic_wire"], str]
    """
    Specifies the network rails to be used. If not set, will default to the PaymentMethod's preferred network. See the [docs](https://docs.stripe.com/treasury/money-movement/timelines) to learn more about money movement timelines for each network type.
    """


class ReceivedCreditCreateParamsInitiatingPaymentMethodDetails(TypedDict):
    type: Literal["us_bank_account"]
    """
    The source type.
    """
    us_bank_account: NotRequired[
        "ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount"
    ]
    """
    Optional fields for `us_bank_account`.
    """


class ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount(
    TypedDict,
):
    account_holder_name: NotRequired[str]
    """
    The bank account holder's name.
    """
    account_number: NotRequired[str]
    """
    The bank account number.
    """
    routing_number: NotRequired[str]
    """
    The bank account's routing number.
    """



"""
We use a combination of generics and inheritance to construct 4 handler classes with perfect type information while reusing as much code as we can.
Each handler has the methods/signatures that will actually work:

|       | verified                            | unverified                                             |
| ----- | ----------------------------------- | ------------------------------------------------------ |
| sync  | StripeEventNotificationHandler      | StripeEventNotificationHandlerWithoutVerification      |
| async | AsyncStripeEventNotificationHandler | AsyncStripeEventNotificationHandlerWithoutVerification |

A pair of generic variables gives us the following class hierarchy (names edited for brevity):

- _BaseHandler(Generic[CallbackReturn, PreHandleReturn])
    - _SyncHandler(_BaseHandler[None, bool])
        - StripeHandler(_SyncHandler)
        - StripeHandlerWithoutVerification(_SyncHandler)
    - _AsyncHandler(_BaseHandler[None, bool])
        - AsyncStripeHandler(_AsyncHandler)
        - AsyncStripeHandlerWithoutVerification(_AsyncHandler)

Each defines `handle` and `register` methods corresponding to the types it expects
"""

from dataclasses import dataclass
from typing_extensions import TYPE_CHECKING, Awaitable

from typing import (
    Callable,
    Generic,
    List,
    Optional,
    TypeVar,
)

# Import at runtime for isinstance check and type annotations
from stripe.v2.core._event import EventNotification, UnknownEventNotification
from stripe._webhook import WebhookPayload

if TYPE_CHECKING:
    from stripe._stripe_client import StripeClient

    # event-notification-types: The beginning of the section generated from our OpenAPI spec
    from stripe.events._v1_billing_meter_error_report_triggered_event import (
        V1BillingMeterErrorReportTriggeredEventNotification,
    )
    from stripe.events._v1_billing_meter_no_meter_found_event import (
        V1BillingMeterNoMeterFoundEventNotification,
    )
    from stripe.events._v2_commerce_product_catalog_imports_failed_event import (
        V2CommerceProductCatalogImportsFailedEventNotification,
    )
    from stripe.events._v2_commerce_product_catalog_imports_processing_event import (
        V2CommerceProductCatalogImportsProcessingEventNotification,
    )
    from stripe.events._v2_commerce_product_catalog_imports_succeeded_event import (
        V2CommerceProductCatalogImportsSucceededEventNotification,
    )
    from stripe.events._v2_commerce_product_catalog_imports_succeeded_with_errors_event import (
        V2CommerceProductCatalogImportsSucceededWithErrorsEventNotification,
    )
    from stripe.events._v2_core_account_closed_event import (
        V2CoreAccountClosedEventNotification,
    )
    from stripe.events._v2_core_account_created_event import (
        V2CoreAccountCreatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_customer_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_customer_updated_event import (
        V2CoreAccountIncludingConfigurationCustomerUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_merchant_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_merchant_updated_event import (
        V2CoreAccountIncludingConfigurationMerchantUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_recipient_capability_status_updated_event import (
        V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_configuration_recipient_updated_event import (
        V2CoreAccountIncludingConfigurationRecipientUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_defaults_updated_event import (
        V2CoreAccountIncludingDefaultsUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_future_requirements_updated_event import (
        V2CoreAccountIncludingFutureRequirementsUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_identity_updated_event import (
        V2CoreAccountIncludingIdentityUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_including_requirements_updated_event import (
        V2CoreAccountIncludingRequirementsUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_link_returned_event import (
        V2CoreAccountLinkReturnedEventNotification,
    )
    from stripe.events._v2_core_account_person_created_event import (
        V2CoreAccountPersonCreatedEventNotification,
    )
    from stripe.events._v2_core_account_person_deleted_event import (
        V2CoreAccountPersonDeletedEventNotification,
    )
    from stripe.events._v2_core_account_person_updated_event import (
        V2CoreAccountPersonUpdatedEventNotification,
    )
    from stripe.events._v2_core_account_updated_event import (
        V2CoreAccountUpdatedEventNotification,
    )
    from stripe.events._v2_core_event_destination_ping_event import (
        V2CoreEventDestinationPingEventNotification,
    )
    # event-notification-types: The end of the section generated from our OpenAPI spec

# internal type to represent any EventNotification subclass
EventNotificationChild = TypeVar(
    "EventNotificationChild", bound="EventNotification"
)


@dataclass
class UnhandledNotificationDetails:
    """
    Information about an unhandled event notification to make it easier to respond (and potentially update your integration).
    """

    is_known_event_type: bool
    """
    If true, the unhandled event's type is known to the SDK (i.e., it was successfully deserialized into a specific `EventNotification` subclass).
    """


# The handler base class is generic over what its callbacks return, which lets us reuse base classes for both the sync and async handlers.
CallbackReturn = TypeVar("CallbackReturn")
PreHandleReturn = TypeVar("PreHandleReturn")

_FallbackCallback = Callable[
    [EventNotification, "StripeClient", UnhandledNotificationDetails],
    CallbackReturn,
]

_PreHandleCallback = Callable[
    [EventNotification, "StripeClient"], PreHandleReturn
]

FallbackCallback = _FallbackCallback[None]
"""
Called when no other callback is registered for a given event notification type.
"""

AsyncFallbackCallback = _FallbackCallback[Awaitable[None]]
"""
This async function is called when no other callback is registered for a given event notification type.
"""

PreHandleCallback = _PreHandleCallback[bool]
"""
Called before any of your callbacks are run. Useful for filtering.
"""

AsyncPreHandleCallback = _PreHandleCallback[Awaitable[bool]]
"""
This async function is called before any of your callbacks are run. Useful for filtering.
"""


class _BaseEventNotificationHandler(Generic[CallbackReturn, PreHandleReturn]):
    """
    Shared internal registration machinery for the user-facing event handlers.

    Holds everything that doesn't depend on whether callbacks are awaited; the
    sync and async subclasses below add only the dispatch loop itself.
    """

    def __init__(
        self,
        client: "StripeClient",
        fallback_callback: _FallbackCallback[CallbackReturn],
    ) -> None:
        self._registered_handlers = {}
        self._client = client
        self.fallback_callback = fallback_callback
        # once this is true, adding additional handlers results in an error
        self._has_handled_events = False
        self._pre_handle_callback: Optional[
            _PreHandleCallback[PreHandleReturn]
        ] = None

    def _assert_hasnt_handled(self) -> None:
        """
        Callbacks are expected to be registered on startup, so registering anything after handling an event indicates a bug.
        """
        if self._has_handled_events:
            raise RuntimeError(
                "Cannot register new callbacks after an event has been handled. This is indicative of a bug."
            )

    def pre_handle(
        self, func: _PreHandleCallback[PreHandleReturn]
    ) -> _PreHandleCallback[PreHandleReturn]:
        """
        Registers a function that will be run before any event-specific callbacks. A useful place to store event-agnostic logic, such as logging or checking for [duplicate event deliveries](https://docs.stripe.com/webhooks#handle-duplicate-events).

        Returning `True` causes handling to continue as normal; returning `False` returns from `.handle()` immediately, so neither the registered callback nor the fallback callback are called.
        """
        self._assert_hasnt_handled()
        if self._pre_handle_callback:
            raise ValueError("A pre_handle callback is already registered")

        self._pre_handle_callback = func
        return func

    def _callback_for(self, event_notif: "EventNotification"):
        """
        Returns the callback registered for this event's type, if any.
        """
        return self._registered_handlers.get(event_notif.type)

    def _unhandled_details(
        self, event_notif: "EventNotification"
    ) -> UnhandledNotificationDetails:
        return UnhandledNotificationDetails(
            is_known_event_type=not isinstance(
                event_notif, UnknownEventNotification
            )
        )

    def _register(
        self,
        event_type: str,
        func: "Callable[[EventNotificationChild, StripeClient], CallbackReturn]",
    ) -> None:
        self._assert_hasnt_handled()
        if event_type in self._registered_handlers:
            raise ValueError(
                f'Callback for event type "{event_type}" is already registered'
            )

        self._registered_handlers[event_type] = func

    @property
    def registered_event_types(self) -> List[str]:
        """
        Returns an alphabetized list of all event types that have registered handlers.
        """
        return sorted(self._registered_handlers.keys())

    # event-notification-registration-methods: The beginning of the section generated from our OpenAPI spec
    def on_v1_billing_meter_error_report_triggered(
        self,
        func: "Callable[[V1BillingMeterErrorReportTriggeredEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V1BillingMeterErrorReportTriggeredEvent` (`v1.billing.meter.error_report_triggered`) event notification.
        """
        self._register(
            "v1.billing.meter.error_report_triggered",
            func,
        )
        return func

    def on_v1_billing_meter_no_meter_found(
        self,
        func: "Callable[[V1BillingMeterNoMeterFoundEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V1BillingMeterNoMeterFoundEvent` (`v1.billing.meter.no_meter_found`) event notification.
        """
        self._register(
            "v1.billing.meter.no_meter_found",
            func,
        )
        return func

    def on_v2_commerce_product_catalog_imports_failed(
        self,
        func: "Callable[[V2CommerceProductCatalogImportsFailedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CommerceProductCatalogImportsFailedEvent` (`v2.commerce.product_catalog.imports.failed`) event notification.
        """
        self._register(
            "v2.commerce.product_catalog.imports.failed",
            func,
        )
        return func

    def on_v2_commerce_product_catalog_imports_processing(
        self,
        func: "Callable[[V2CommerceProductCatalogImportsProcessingEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CommerceProductCatalogImportsProcessingEvent` (`v2.commerce.product_catalog.imports.processing`) event notification.
        """
        self._register(
            "v2.commerce.product_catalog.imports.processing",
            func,
        )
        return func

    def on_v2_commerce_product_catalog_imports_succeeded(
        self,
        func: "Callable[[V2CommerceProductCatalogImportsSucceededEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CommerceProductCatalogImportsSucceededEvent` (`v2.commerce.product_catalog.imports.succeeded`) event notification.
        """
        self._register(
            "v2.commerce.product_catalog.imports.succeeded",
            func,
        )
        return func

    def on_v2_commerce_product_catalog_imports_succeeded_with_errors(
        self,
        func: "Callable[[V2CommerceProductCatalogImportsSucceededWithErrorsEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CommerceProductCatalogImportsSucceededWithErrorsEvent` (`v2.commerce.product_catalog.imports.succeeded_with_errors`) event notification.
        """
        self._register(
            "v2.commerce.product_catalog.imports.succeeded_with_errors",
            func,
        )
        return func

    def on_v2_core_account_closed(
        self,
        func: "Callable[[V2CoreAccountClosedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountClosedEvent` (`v2.core.account.closed`) event notification.
        """
        self._register(
            "v2.core.account.closed",
            func,
        )
        return func

    def on_v2_core_account_created(
        self,
        func: "Callable[[V2CoreAccountCreatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountCreatedEvent` (`v2.core.account.created`) event notification.
        """
        self._register(
            "v2.core.account.created",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_customer_capability_status_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationCustomerCapabilityStatusUpdatedEvent` (`v2.core.account[configuration.customer].capability_status_updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.customer].capability_status_updated",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_customer_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationCustomerUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationCustomerUpdatedEvent` (`v2.core.account[configuration.customer].updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.customer].updated",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_merchant_capability_status_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEvent` (`v2.core.account[configuration.merchant].capability_status_updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.merchant].capability_status_updated",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_merchant_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationMerchantUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationMerchantUpdatedEvent` (`v2.core.account[configuration.merchant].updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.merchant].updated",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_recipient_capability_status_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationRecipientCapabilityStatusUpdatedEvent` (`v2.core.account[configuration.recipient].capability_status_updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.recipient].capability_status_updated",
            func,
        )
        return func

    def on_v2_core_account_including_configuration_recipient_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingConfigurationRecipientUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingConfigurationRecipientUpdatedEvent` (`v2.core.account[configuration.recipient].updated`) event notification.
        """
        self._register(
            "v2.core.account[configuration.recipient].updated",
            func,
        )
        return func

    def on_v2_core_account_including_defaults_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingDefaultsUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingDefaultsUpdatedEvent` (`v2.core.account[defaults].updated`) event notification.
        """
        self._register(
            "v2.core.account[defaults].updated",
            func,
        )
        return func

    def on_v2_core_account_including_future_requirements_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingFutureRequirementsUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingFutureRequirementsUpdatedEvent` (`v2.core.account[future_requirements].updated`) event notification.
        """
        self._register(
            "v2.core.account[future_requirements].updated",
            func,
        )
        return func

    def on_v2_core_account_including_identity_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingIdentityUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingIdentityUpdatedEvent` (`v2.core.account[identity].updated`) event notification.
        """
        self._register(
            "v2.core.account[identity].updated",
            func,
        )
        return func

    def on_v2_core_account_including_requirements_updated(
        self,
        func: "Callable[[V2CoreAccountIncludingRequirementsUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountIncludingRequirementsUpdatedEvent` (`v2.core.account[requirements].updated`) event notification.
        """
        self._register(
            "v2.core.account[requirements].updated",
            func,
        )
        return func

    def on_v2_core_account_link_returned(
        self,
        func: "Callable[[V2CoreAccountLinkReturnedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountLinkReturnedEvent` (`v2.core.account_link.returned`) event notification.
        """
        self._register(
            "v2.core.account_link.returned",
            func,
        )
        return func

    def on_v2_core_account_person_created(
        self,
        func: "Callable[[V2CoreAccountPersonCreatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountPersonCreatedEvent` (`v2.core.account_person.created`) event notification.
        """
        self._register(
            "v2.core.account_person.created",
            func,
        )
        return func

    def on_v2_core_account_person_deleted(
        self,
        func: "Callable[[V2CoreAccountPersonDeletedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountPersonDeletedEvent` (`v2.core.account_person.deleted`) event notification.
        """
        self._register(
            "v2.core.account_person.deleted",
            func,
        )
        return func

    def on_v2_core_account_person_updated(
        self,
        func: "Callable[[V2CoreAccountPersonUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountPersonUpdatedEvent` (`v2.core.account_person.updated`) event notification.
        """
        self._register(
            "v2.core.account_person.updated",
            func,
        )
        return func

    def on_v2_core_account_updated(
        self,
        func: "Callable[[V2CoreAccountUpdatedEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreAccountUpdatedEvent` (`v2.core.account.updated`) event notification.
        """
        self._register(
            "v2.core.account.updated",
            func,
        )
        return func

    def on_v2_core_event_destination_ping(
        self,
        func: "Callable[[V2CoreEventDestinationPingEventNotification, StripeClient], CallbackReturn]",
    ):
        """
        Registers a callback for the `V2CoreEventDestinationPingEvent` (`v2.core.event_destination.ping`) event notification.
        """
        self._register(
            "v2.core.event_destination.ping",
            func,
        )
        return func

    # event-notification-registration-methods: The end of the section generated from our OpenAPI spec


class _SyncEventNotificationHandler(_BaseEventNotificationHandler[None, bool]):
    """
    Adds synchronous dispatch. Shared by the verifying and non-verifying sync
    handlers, which differ only in how they parse the incoming payload.
    """

    def _dispatch(self, event_notif: "EventNotification") -> None:
        client = self._client.with_stripe_context(event_notif.context)

        if self._pre_handle_callback and not self._pre_handle_callback(
            event_notif, client
        ):
            return

        if callback := self._callback_for(event_notif):
            callback(event_notif, client)
        else:
            self.fallback_callback(
                event_notif, client, self._unhandled_details(event_notif)
            )


class _AsyncEventNotificationHandler(
    _BaseEventNotificationHandler[Awaitable[None], Awaitable[bool]]
):
    """
    Adds asynchronous dispatch. Only the callbacks are awaited: verifying a
    signature and parsing the payload are pure CPU work, so they stay
    synchronous even here.
    """

    async def _dispatch_async(self, event_notif: "EventNotification") -> None:
        client = self._client.with_stripe_context(event_notif.context)

        if self._pre_handle_callback and not await self._pre_handle_callback(
            event_notif, client
        ):
            return

        if callback := self._callback_for(event_notif):
            await callback(event_notif, client)
        else:
            await self.fallback_callback(
                event_notif, client, self._unhandled_details(event_notif)
            )


class StripeEventNotificationHandler(_SyncEventNotificationHandler):
    """
    An on-rails experience for handling Stripe event notifications. Define callbacks for individual event types and an instance of this class will be responsible for verifying and routing the event.
    """

    def __init__(
        self,
        client: "StripeClient",
        webhook_secret: Optional[str],
        fallback_callback: FallbackCallback,
    ) -> None:
        """`webhook_secret` is only marked as `Optional` so it plays nicely with the types commonly returned from web frameworks. This raises a `ValueError` if a secret is not provided."""
        super().__init__(client, fallback_callback)
        if not webhook_secret:
            raise ValueError("webhook_secret must be a non-empty string")
        self._webhook_secret = webhook_secret

    def handle(self, webhook_body: WebhookPayload, sig_header: Optional[str]):
        """
        Process an incoming webhook, routing it to the correct registered callback (or your fallback).

        `sig_header` is only marked as `Optional` so it plays nicely with the types commonly returned from web frameworks. This raises a `SignatureVerificationError` if a signature is not provided.
        """
        # set before parsing, so that even a failed parse locks out registration.
        # modification isn't thread-safe, but we expect callbacks to get registered synchronously at startup
        # making a race condition here unlikely
        self._has_handled_events = True

        event_notif = self._client.parse_event_notification(
            webhook_body, sig_header, self._webhook_secret
        )

        self._dispatch(event_notif)

    @staticmethod
    def without_verification(
        client: "StripeClient",
        fallback_callback: FallbackCallback,
    ) -> "StripeEventNotificationHandlerWithoutVerification":
        return StripeEventNotificationHandlerWithoutVerification(
            client, fallback_callback
        )


class StripeEventNotificationHandlerWithoutVerification(
    _SyncEventNotificationHandler
):
    """
    A variant of StripeEventNotificationHandler that parses events without verifying webhook signatures. Intended for pre-authenticated channels like AWS EventBridge, Azure Event Grid, or your own pre-authenticated queuing system.

    Prefer `StripeEventNotificationHandler.without_verification()` or `client.notification_handler_without_verification()` instead of constructing it directly.
    """

    def handle(self, webhook_body: WebhookPayload):
        """
        Process an incoming webhook, routing it to the correct registered callback (or your fallback) without signature verification.
        """
        self._has_handled_events = True

        event_notif = (
            self._client.parse_event_notification_without_verification(
                webhook_body
            )
        )

        self._dispatch(event_notif)


class AsyncStripeEventNotificationHandler(_AsyncEventNotificationHandler):
    """
    The async equivalent of `StripeEventNotificationHandler`, for use from async web frameworks. Register `async def` callbacks and await `.handle_async()`.
    """

    def __init__(
        self,
        client: "StripeClient",
        webhook_secret: Optional[str],
        fallback_callback: AsyncFallbackCallback,
    ) -> None:
        """`webhook_secret` is only marked as `Optional` so it plays nicely with the types commonly returned from web frameworks. This raises a `ValueError` if a secret is not provided."""
        super().__init__(client, fallback_callback)
        if not webhook_secret:
            raise ValueError("webhook_secret must be a non-empty string")
        self._webhook_secret = webhook_secret

    async def handle_async(
        self, webhook_body: WebhookPayload, sig_header: Optional[str]
    ):
        """
        Process an incoming webhook, routing it to the correct registered callback (or your fallback).

        `sig_header` is only marked as `Optional` so it plays nicely with the types commonly returned from web frameworks. This raises a `SignatureVerificationError` if a signature is not provided.
        """
        self._has_handled_events = True

        event_notif = self._client.parse_event_notification(
            webhook_body, sig_header, self._webhook_secret
        )

        await self._dispatch_async(event_notif)

    @staticmethod
    def without_verification(
        client: "StripeClient",
        fallback_callback: AsyncFallbackCallback,
    ) -> "AsyncStripeEventNotificationHandlerWithoutVerification":
        return AsyncStripeEventNotificationHandlerWithoutVerification(
            client, fallback_callback
        )


class AsyncStripeEventNotificationHandlerWithoutVerification(
    _AsyncEventNotificationHandler
):
    """
    A variant of AsyncStripeEventNotificationHandler that parses events without verifying webhook signatures. Intended for pre-authenticated channels like AWS EventBridge, Azure Event Grid, or your own pre-authenticated queuing system.

    Prefer `AsyncStripeEventNotificationHandler.without_verification()` or `client.async_notification_handler_without_verification()` instead of constructing it directly.
    """

    async def handle_async(self, webhook_body: WebhookPayload):
        """
        Process an incoming webhook, routing it to the correct registered callback (or your fallback) without signature verification.
        """
        self._has_handled_events = True

        event_notif = (
            self._client.parse_event_notification_without_verification(
                webhook_body
            )
        )

        await self._dispatch_async(event_notif)
